/*
 * Interrupt driven serial driver
 */
#include <avr/signal.h>
#include <stdlib.h>
#include <avr/io.h>
#include <stdio.h>
#include <string.h>
#include <avr/interrupt.h>

#include "serial.h"

// size of receive buffer
#define UART_BUFSIZE 16

char *uart_rx_buffer=0;
volatile int  uart_rx=0;
volatile int  uart_line_received=0;

/*
 * initialize uart
 */
void UartInit(unsigned long rate)
{
  unsigned int sv;
  if (!uart_rx_buffer) uart_rx_buffer=malloc(UART_BUFSIZE);
  uart_rx=0;

  rate <<= 3;
  sv = (unsigned int) ((CPUCLOCK / rate + 1UL) / 2UL) - 1;
  UBRR=(unsigned char) sv;

  UCR= _BV(RXCIE) | _BV(RXEN) | _BV(TXEN);
  // dummy read to reset error bits
  sv=UDR;
	fdevopen(UartPutChar, 0 /*UartGetChar*/, 0);
}

/*
 * transmit a character
 */
int UartPutChar(char ch)
{
	// insert a LF if there is a CR
	if (ch=='\n') UartPutChar('\r');
	// wait for transmitter empty
  loop_until_bit_is_set(USR, UDRE);
	// transmit character
	UDR=ch;
	return 0;
}

/*
 * get string from uart
 */
int UartGetString(char *s)
{
	if (!uart_line_received) return 0;
	strcpy(s,uart_rx_buffer);
	uart_rx=0;
	uart_line_received=0;
	return 1;
}


SIGNAL(SIG_UART_RECV)
{
	char ch;
	
  // read the character
  ch = UDR;
  sei();
  
  // linebuffer ready?
  if (!uart_line_received)
  {
  	if (ch==0x0d)
  	{
  		uart_rx_buffer[uart_rx]=0;
  		uart_line_received=1;
  	}
  	else
  	  if (ch!=0x0a)
  	  {
        if ((ch==0x08) || (ch==0x7f))
        {
          if (uart_rx>0)
          {
            uart_rx_buffer[--uart_rx]=0;
// Echo 
//            UartPutChar(ch);
//            UartPutChar(' ');
//            UartPutChar(ch);
          }
          else
            UartPutChar(7);
        }  
        else
        {
  	      uart_rx_buffer[uart_rx]=ch;
  	      uart_rx++;
  	      if (uart_rx>UART_BUFSIZE-1)
  	      {
  	    	  uart_rx_buffer[uart_rx]=0;
  	    	  uart_line_received=1;
  	      }
// Echo  	      
//  	      UartPutChar(ch);
  	    }  
  	  }
  }
  else
    UartPutChar(7);  
}
